using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Application = System.Windows.Forms.Application;
using Form = System.Windows.Forms.Form;
using Model;
using View;
using Blaze_Brigade;
using System.Diagnostics;
using System.Threading;

/// <summary>
/// The controller in MVC. These classes will control how the Model is used, and how the 
/// View will be displayed to the user.
/// </summary>
namespace Controller

{
    /// <summary>
    /// Main Controller for game
    /// </summary>
    public class Game : Microsoft.Xna.Framework.Game
    {

        #region Variables

        static Game instance;

        public static Game Instance
        {
            get { return instance; }
        }

        GameMenuState currentGameState = GameMenuState.MainMenu; // game starts in main menu screen
        MainMenu mMenu; // main menu variable
        View.HowToPlay tut; // instruction screen variable
        View.HowToPlay2 tut2; // instruction 2 screen variable
        View.HowToPlay3 tut3; // instruction 3 screen variable
        Graph graph;
        Player player1;
        Player player2;

        Camera camera;
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Texture2D backGround, moveableNode, attackableNode, gameOver, endTurnButton, player1Transition, player2Transition;
        private SpriteFont font; // custom font
        private SpriteFont largeFont; // custom font 2
        private SpriteFont largestFont; // custom font 3
        private float ElapsedTime = 0f;
        private float damageDealtTime1 = 0f;
        private float damageDealtTime2 = 0f;

        #endregion

        // constructor for game
        public Game()
        {
            instance = this;

            graphics = new GraphicsDeviceManager(this);
            graphics.PreferredBackBufferWidth = GameState.SCREEN_HEIGHT;
            graphics.PreferredBackBufferHeight = GameState.SCREEN_WIDTH;
            Content.RootDirectory = "Content";

            //instantiate the game windows
            mMenu = new MainMenu();
            tut = new HowToPlay();
            tut2 = new HowToPlay2();
            tut3 = new HowToPlay3();
        }

        /**
        initializes game
        */
        protected override void Initialize()
        {
            Form MyGameForm = (Form)Form.FromHandle(Window.Handle); // creates handle
            MyGameForm.Opacity = 0; // hides the game window until it's set to visible
            base.Initialize();
        }
        /**
        loads required textures and values for starting game
        */
        protected override void LoadContent()
        {
            // creates a new SpriteBatch, which can be used to draw textures
            spriteBatch = new SpriteBatch(GraphicsDevice);

            initializeGame();

            backGround = Content.Load<Texture2D>("Game_Map"); // load background
            moveableNode = Content.Load<Texture2D>("moveableNode");
            attackableNode = Content.Load<Texture2D>("attackableNode");
            gameOver = Content.Load<Texture2D>("exit_game");
            endTurnButton = Content.Load<Texture2D>("End_Button");
            player1Transition = Content.Load<Texture2D>("player1turn");
            player2Transition = Content.Load<Texture2D>("player2turn");

            font = Content.Load<SpriteFont>("PixelFont"); //loads font PixelFont
            largeFont = Content.Load<SpriteFont>("PixelFontLarge"); //loads font PixelFont
            largestFont = Content.Load<SpriteFont>("PixelFontLargest"); //loads font PixelFont

            graphics.PreferredBackBufferWidth = GameState.SCREEN_WIDTH; // width of screen
            graphics.PreferredBackBufferHeight = GameState.SCREEN_HEIGHT; //height of screen

            IsMouseVisible = true; // sets mouse visibility to true
            graphics.ApplyChanges(); // load images
        }

        //initializes players and units
        private void initializeGame()
        {
            camera = new View.Camera();

            graph = new Graph(50, 32);  // create graph

            // set players
            player1 = new Player();
            player2 = new Player();
            GameState.currentPlayer = (player1);
            GameState.enemyPlayer = (player2);

            // load character sprite and set position
            player1.addUnit(getNewUnit(UnitType.Warrior, new Vector2(4 * 32f, 6 * 32f), player1));
            player1.addUnit(getNewUnit(UnitType.Mage, new Vector2(4 * 32f, 10 * 32f), player1));
            player1.addUnit(getNewUnit(UnitType.Archer, new Vector2(4 * 32f, 8 * 32f), player1));
            player2.addUnit(getNewUnit(UnitType.Warrior, new Vector2(8 * 32f, 8 * 32f), player2));
            player2.addUnit(getNewUnit(UnitType.Mage, new Vector2(8 * 32f, 6 * 32f), player2));
            player2.addUnit(getNewUnit(UnitType.Archer, new Vector2(8 * 32f, 10 * 32f), player2));
            setObstacles(graph);

        }

        /** 
        Updates game in real time - 60fps
        */
        protected override void Update(GameTime gameTime)
        {
            #region Exiting Game
            // allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == Microsoft.Xna.Framework.Input.ButtonState.Pressed)
                this.Exit();

            if (this.IsActive)
            {
                // call mouse handler if game window is active
                MouseHandler.updateMouse(graph, camera);
            }
            #endregion

            if (GameState.exitGameClicked)
            {
                this.Exit();
            }

            #region Game States
            MouseState mouse = Mouse.GetState();
            //Lists possible game states
            switch (currentGameState)
            {
                case GameMenuState.MainMenu: // if mouse is clicked... call method
                    mMenu.Show(); // show main menu
                    if (mMenu.start == true) //if New game is selected
                    {
                        mMenu.Hide(); //close Main Menu screen
                        tut.Hide(); //close How To Play Menu
                        Form GameForm = (Form)Form.FromHandle(Window.Handle);
                        GameForm.Opacity = 100; // make screen show
                        currentGameState = GameMenuState.Playing; //set game state to Playing
                        break;
                    }
                    if (mMenu.getInstruct() == true) //if How to Play is selected
                    {
                        mMenu.Hide(); //hide visibility of menu window
                        currentGameState = GameMenuState.HowToPlay; //change game state to How To Play
                        break;
                    }
                    if (mMenu.quit == true) // if quit clicked
                    {
                        this.Exit();
                        break;
                    }
                    break;

                case GameMenuState.HowToPlay: // if true.. load new image...
                    tut.Show(); //set visibility of how to play window to true
                    mMenu.setInstructFalse();
                    if (tut.getNext() == true) //if How to Play is selected
                    {
                        tut.Hide(); //hide visibility of menu window
                        currentGameState = GameMenuState.HowToPlay2; //change game state to How To Play
                        break;
                    }
                    if (tut.getQuit() == true) // if quit clicked
                    {
                        tut.setQuitFalse();
                        tut.Hide(); //hide 
                        currentGameState = GameMenuState.MainMenu; //changes state back to main menu
                        break;
                    }
                    break;

                case GameMenuState.HowToPlay2: // if true.. load new image...
                    tut2.Show(); //set visibility of how to play window to true
                    tut.setNextFalse();
                    if (tut2.getNext() == true) //if How to Play is selected
                    {
                        tut2.Hide(); //hide visibility of menu window
                        currentGameState = GameMenuState.HowToPlay3; //change game state to How To Play
                        break;
                    }
                    if (tut2.getQuit() == true) // if quit clicked
                    {
                        tut2.setQuitFalse();
                        tut2.Hide(); //hide 
                        currentGameState = GameMenuState.HowToPlay; //changes state back to main menu
                        break;
                    }
                    break;

                case GameMenuState.HowToPlay3: // if true.. load new image...
                    tut3.Show(); //set visibility of how to play window to true
                    tut2.setNextFalse();
                    if (tut3.getQuit() == true) // if quit clicked
                    {
                        tut3.setQuitFalse();
                        tut3.Hide(); //hide 
                        currentGameState = GameMenuState.HowToPlay2; //changes state back to main menu
                        break;
                    }
                    break;

                case GameMenuState.Playing: // if true.. load new image...
                    backGround = Content.Load<Texture2D>("Game_Map"); // load background

                    if (GameFunction.isTurnOver())
                    {
                        Player tempPlayer = GameState.currentPlayer;
                        GameState.currentPlayer = (GameState.enemyPlayer);
                        GameState.enemyPlayer = (tempPlayer);
                        GameState.transitionTurn = true;
                        GameFunction.startTurn(GameState.currentPlayer);
                    }

                    if (GameState.transitionTurn)
                    {
                        ElapsedTime += (float)gameTime.ElapsedGameTime.TotalSeconds;
                        if (ElapsedTime > 1.5)
                        {
                            GameState.transitionTurn = false;
                            ElapsedTime = 0f;
                        }
                    }

                    if (GameState.currentPlayerDamagePopup)
                    {
                        damageDealtTime1 += (float)gameTime.ElapsedGameTime.TotalSeconds;
                        if (damageDealtTime1 > 2.5)
                        {
                            GameState.currentPlayerDamagePopup = false;
                            damageDealtTime1 = 0f;
                        }
                    }

                    if (GameState.enemyPlayerDamagePopup)
                    {
                        damageDealtTime2 += (float)gameTime.ElapsedGameTime.TotalSeconds;
                        if (damageDealtTime2 > 2.5)
                        {
                            GameState.enemyPlayerDamagePopup = false;
                            damageDealtTime2 = 0f;
                        }
                    }

                    // removes deceased units in Player 1
                    foreach (Unit unit in player1.getUnits())
                    {
                        if (!unit.Alive)
                        {
                            GameFunction.removeUnit(graph, player1, unit);
                            break;
                        }
                        if (GameFunction.isGameOver(player1, player2))
                        {
                            GameState.gameOver = true;
                            break;
                        }
                    }

                    // removes deceased units in Player 2
                    foreach (Unit unit in player2.getUnits())
                    {
                        if (!unit.Alive)
                        {
                            GameFunction.removeUnit(graph, player2, unit);
                            break;
                        }
                        if (GameFunction.isGameOver(player1, player2))
                        {
                            GameState.gameOver = true;
                            break;
                        }
                    }
                    break;
            }
            #endregion

            base.Update(gameTime); // repeatedly calls update
        }

        /**
        draws the game as it updates
        */
        protected override void Draw(GameTime gameTime)
        {
            // draw elements that are affected by camera (scrollable)
            spriteBatch.Begin(SpriteSortMode.BackToFront, BlendState.AlphaBlend, null, null, null, null, camera.TransformMatrix);

            //only draw objects relevent to current game state
            switch (currentGameState)
            {
                case GameMenuState.Playing: // while in game

                    #region Player 1 Units
                    // draws units for player 1
                    for (int i = 0; i < player1.getNumOfUnits(); i++)
                    {
                        Unit unit = player1.getUnits().ElementAt(i); //gets unit at i
                        if (unit.Alive)
                        {
                            spriteBatch.Draw(unit.getSpriteImage(), unit.PixelCoordinates,
                                unit.getCurrentFrame(), Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.8f);
                        }
                    }
                    #endregion

                    #region Player 2 Units
                    // draws units for player 2
                    for (int i = 0; i < player2.getNumOfUnits(); i++)
                    {
                        Unit unit = player2.getUnits().ElementAt(i);
                        if (unit.Alive)
                        {
                            spriteBatch.Draw(unit.getSpriteImage(), unit.PixelCoordinates,
                                unit.getCurrentFrame(), Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.8f);
                        }
                    }
                    #endregion

                    if (GameState.currentPlayerDamagePopup)
                    {
                        spriteBatch.DrawString(largestFont, GameState.CurrentPlayerDamageDealt, GameState.lastDefendingUnit.PixelCoordinates + (new Vector2(2, -65)), Color.DarkRed, 0,
                            Vector2.Zero, 1f, SpriteEffects.None, 0f); 
                    }
                    if (GameState.enemyPlayerDamagePopup)
                    {
                        spriteBatch.DrawString(largestFont, GameState.EnemyPlayerDamageDealt, GameState.lastAttackingUnit.PixelCoordinates + (new Vector2(2, -65)), Color.DarkRed, 0,
                            Vector2.Zero, 1f, SpriteEffects.None, 0f);
                    }


                    #region When Unit is selected
                    //if unit is currently clicked on
                    if (GameState.playableUnitSelected)
                    {
                        Unit unit = GameState.selectedUnit;

                        if (!GameState.isAnimating)
                        {
                            #region Highlight nodes

                            if (GameState.beforeMove && !GameState.attackSelect) // if unit has yet to move, display the overall move and attack range of unit
                            {
                                // Highlight movable nodes in blue
                                foreach (Node move in GameState.moveableNodes)
                                {
                                    spriteBatch.Draw(moveableNode, move.getPosition(), null, Color.White * 0.2f, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.9f);
                                }

                                // Highlight attackable nodes in red
                                LinkedList<Node> attackableNodes = GameFunction.getAttackableNodes(graph, unit);

                                foreach (Node attack in attackableNodes)
                                {
                                    if ((!GameState.moveableNodes.Contains(attack)) && (attack.unitOnNode != unit) && (!attack.isObstacle))
                                    {
                                        spriteBatch.Draw(attackableNode, attack.getPosition(), null, Color.White * 0.2f, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.9f);
                                    }
                                }

                            }
                            else // else if unit has already moved, only display the attack range
                            {
                                LinkedList<Node> attackableNodes = GameFunction.getAttackRangeAfterMoving(graph, unit);
                                foreach (Node attack in attackableNodes)
                                {
                                    spriteBatch.Draw(attackableNode, attack.getPosition(), null, Color.White * 0.2f, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.9f);
                                }
                            }
                            #endregion

                            #region Drop Down menu
                            if (GameState.dropDownMenuOpen) // if dropDowMenu should be opened, draw dropDownMenu
                            {
                                unit.setButtonCoordinates(unit.PixelCoordinates);

                                Button[] unitButtons = unit.getButtons();
                                for (int i = 0; i < 4; i++)
                                {
                                    if (unitButtons[i].Active)
                                    {
                                        spriteBatch.Draw(unitButtons[i].getImage(), unitButtons[i].getPixelCoordinates(), null, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0f);
                                    }
                                }

                            }

                            #endregion

                            //Inventory Drop down menu - if item is clicked
                            if (GameState.inventoryOpen)
                            {
                                unit.setButtonCoordinates(unit.PixelCoordinates); //update button positions

                                Button[] unitButtons = unit.getButtons(); //for each inventory button
                                for (int i = 5; i < 9; i++)
                                {
                                    if (unitButtons[i].Active) //if inventory menu buttons are active
                                    {
                                        if (unitButtons[i].hasItem) //if current menu button actually has an item stored in it
                                        {
                                            spriteBatch.DrawString(font, unitButtons[i].weapon.name.ToString(), unitButtons[i].getPixelCoordinates() + (new Vector2(15, 5)), Color.Black, 0,
                                                Vector2.Zero, 1f, SpriteEffects.None, 0f); //draws item stored in unitButtons[i]
                                            spriteBatch.Draw(unitButtons[i].getImage(), unitButtons[i].getPixelCoordinates(), null, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.1f);
                                        }
                                    }
                                }
                            }
                        }
                    }

                    #endregion

                    #region Game Over

                    if (GameState.gameOver)
                    {
                        // draws faded units

                        #region Player 1 Units
                        // redraws units for player 1 in darker shade for game over
                        for (int i = 0; i < player1.getNumOfUnits(); i++)
                        {
                            Unit unit = player1.getUnits().ElementAt(i); //gets unit at i
                            if (unit.Alive)
                            {
                                spriteBatch.Draw(unit.getSpriteImage(), unit.PixelCoordinates,
                                    unit.getCurrentFrame(), Color.Black * 0.5f, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.5f);
                            }
                        }
                        #endregion

                        #region Player 2 Units
                        // redraws units for player 2 in darker shade for game over
                        for (int i = 0; i < player2.getNumOfUnits(); i++)
                        {
                            Unit unit = player2.getUnits().ElementAt(i);
                            if (unit.Alive)
                            {
                                spriteBatch.Draw(unit.getSpriteImage(), unit.PixelCoordinates,
                                    unit.getCurrentFrame(), Color.Black * 0.5f, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.5f);
                            }
                        }
                        #endregion
                    }

                    #endregion

                    if (GameState.endTurnButton)
                    {
                        spriteBatch.Draw(endTurnButton, GameState.endTurnButtonLocation, null, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0);
                    }

                    spriteBatch.Draw(backGround, Vector2.Zero, null, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 1);

                    break;
            }

            spriteBatch.End();      // end spriteBatch

            // draw elements not affected by camera (fixed on window)
            spriteBatch.Begin(SpriteSortMode.BackToFront, BlendState.AlphaBlend, null, null, null, null);   // Note: we don't pass in camera matrix

            if (GameState.playableUnitSelected)
            {
                Unit unit = GameState.selectedUnit;

                #region Attack menu

                Button confirmButton = unit.getButtonType(ButtonType.AttackConfirm);
                Vector2 attackInfoLocation2 = new Vector2(519, 0);

                if (GameState.attackConfirmOpen && !GameState.isAnimating)
                {
                    Unit attackedUnit = GameState.unitToAttack;
                    // get attack stats
                    bool attackType = findAttackType(unit);
                    int damageDealt = DamageCalculations.getDamageDealt(unit, attackedUnit, attackType);
                    int hitCount = DamageCalculations.getHitCount(unit, attackedUnit);
                    int hitRate = DamageCalculations.getHitRate(unit, attackedUnit);
                    int critRate = DamageCalculations.getCritRate(unit, attackedUnit);
                    // get enemy counter attack stats
                    attackType = findAttackType(attackedUnit);
                    int damageDealt2 = DamageCalculations.getDamageDealt(attackedUnit, unit, attackType);
                    int hitCount2 = DamageCalculations.getHitCount(attackedUnit, unit);
                    int hitRate2 = DamageCalculations.getHitRate(attackedUnit, unit);
                    int critRate2 = DamageCalculations.getCritRate(attackedUnit, unit);

                    if (GameState.currentPlayer == player1)
                    {
                        spriteBatch.Draw(unit.getCharAttackInfo(), Vector2.Zero, null, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.7f); //draw charAttackInfoBackground texture for current character
                        spriteBatch.DrawString(largeFont, damageDealt.ToString(), new Vector2(180, 458), Color.DarkBlue, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws damage dealt
                        spriteBatch.DrawString(font, " x " + hitCount.ToString(), new Vector2(195, 459), Color.DarkBlue, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws hit count
                        spriteBatch.DrawString(largeFont, hitRate.ToString() + " %", new Vector2(170, 488), Color.DarkBlue, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws hit rate
                        spriteBatch.DrawString(largeFont, critRate.ToString() + " %", new Vector2(170, 518), Color.DarkBlue, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws crit rate
                        spriteBatch.DrawString(largestFont, unit.Hp.ToString(), new Vector2(342, 475), Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws unit health
                        spriteBatch.DrawString(largeFont, unit.equippedWeapon.name.ToString(), new Vector2(40, 348), Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws unit's weapon

                        spriteBatch.Draw(attackedUnit.getCharAttackInfo(), attackInfoLocation2, null, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.7f); //draw charAttackInfoBackground for unit being attacked
                        spriteBatch.DrawString(largestFont, attackedUnit.Hp.ToString(), new Vector2(862, 475), Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws enemy unit health
                        spriteBatch.DrawString(largeFont, attackedUnit.equippedWeapon.name.ToString(), new Vector2(715, 348), Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws unit's weapon

                        if (GameFunction.isEnemyUnitInRange(graph, attackedUnit, unit))
                        {
                            spriteBatch.DrawString(largeFont, damageDealt2.ToString(), new Vector2(700, 458), Color.DarkRed, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws counterattack damage
                            spriteBatch.DrawString(font, " x " + hitCount2.ToString(), new Vector2(715, 459), Color.DarkRed, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws counterattack hit count
                            spriteBatch.DrawString(largeFont, hitRate2.ToString() + " %", new Vector2(690, 488), Color.DarkRed, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws counterattack hit rate
                            spriteBatch.DrawString(largeFont, critRate2.ToString() + " %", new Vector2(690, 518), Color.DarkRed, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws counterattack crit rate
                        }
                    }

                    else
                    {
                        spriteBatch.Draw(attackedUnit.getCharAttackInfo(), Vector2.Zero, null, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.7f); //draw charAttackInfoBackground texture for current character
                        spriteBatch.DrawString(largeFont, damageDealt.ToString(), new Vector2(180, 458), Color.DarkBlue, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws counterattack damage 
                        spriteBatch.DrawString(font, " x " + hitCount.ToString(), new Vector2(195, 459), Color.DarkBlue, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws counterattack hit count
                        spriteBatch.DrawString(largeFont, hitRate.ToString() + " %", new Vector2(170, 488), Color.DarkBlue, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws counterattack hit rate
                        spriteBatch.DrawString(largeFont, critRate.ToString() + " %", new Vector2(170, 518), Color.DarkBlue, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws counterattack crit rate
                        spriteBatch.DrawString(largestFont, attackedUnit.Hp.ToString(), new Vector2(342, 475), Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws enemy unit health
                        spriteBatch.DrawString(largeFont, attackedUnit.equippedWeapon.name.ToString(), new Vector2(40, 348), Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws unit's weapon

                        spriteBatch.Draw(unit.getCharAttackInfo(), attackInfoLocation2, null, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.7f); //draw charAttackInfoBackground for current character
                        spriteBatch.DrawString(largestFont, unit.Hp.ToString(), new Vector2(862, 475), Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws unit health 
                        spriteBatch.DrawString(largeFont, unit.equippedWeapon.name.ToString(), new Vector2(715, 348), Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws unit's weapon

                        if (GameFunction.isEnemyUnitInRange(graph, unit, attackedUnit))
                        {
                            spriteBatch.DrawString(largeFont, damageDealt.ToString(), new Vector2(700, 458), Color.DarkRed, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws damage
                            spriteBatch.DrawString(font, " x " + hitCount.ToString(), new Vector2(715, 459), Color.DarkRed, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws hit count
                            spriteBatch.DrawString(largeFont, hitRate.ToString() + " %", new Vector2(690, 488), Color.DarkRed, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws hit rate
                            spriteBatch.DrawString(largeFont, critRate.ToString() + " %", new Vector2(690, 518), Color.DarkRed, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws crit rate
                        }
                    }
                    spriteBatch.Draw(confirmButton.getImage(), confirmButton.getPixelCoordinates(), null, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.8f);
                }

                #endregion

                #region Character Info Screen player1

                //if player 1, prints info screen for player 1
                if ((GameState.currentPlayer == player1) && (!GameState.attackConfirmOpen))
                {
                    Vector2 statLocation = new Vector2(170, 535);   // starting location for first stat
                    Vector2 statLocation2 = new Vector2(235, 535);  // starting location for first stat
                    Vector2 increment = new Vector2(0, 20);         // increment downwards for each stat
                    Vector2 infoLocation = new Vector2(20, 513);
                    int[] stats = unit.getStats();

                    for (int k = 0; k < 4; k++) //for stats - level, str, int, skill,
                    {
                        spriteBatch.DrawString(font, stats[k].ToString(), statLocation, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f);
                        statLocation = statLocation + increment; //increment downwards
                    }
                    for (int t = 4; t < 7; t++) //for stats - speed, defense, resistance 
                    {
                        spriteBatch.DrawString(font, stats[t].ToString(), statLocation2, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f);
                        statLocation2 = statLocation2 + increment; //increment downwards
                    }

                    spriteBatch.DrawString(largeFont, unit.Hp.ToString(), new Vector2(278, 532), Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws unit hp
                    spriteBatch.Draw(unit.getCharInfo(), infoLocation, null, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.7f); //draw charInfoBackground texture
                }
                //else, info screen for player 2
                else
                {
                    if (!GameState.attackConfirmOpen)
                    {
                        Vector2 statLocation = new Vector2(795, 533); //starting location for first stat
                        Vector2 statLocation2 = new Vector2(860, 533); //starting location for first stat
                        Vector2 increment = new Vector2(0, 20); //increment downwards for each stat
                        Vector2 infoLocation = new Vector2(635, 513);
                        int[] stats = unit.getStats();

                        for (int k = 0; k < 4; k++) //for stats - level, str, int, skill,
                        {
                            spriteBatch.DrawString(font, stats[k].ToString(), statLocation, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f);
                            statLocation = statLocation + increment; //increment downwards
                        }
                        for (int t = 4; t < 7; t++) //for stats - speed, defense, resistance 
                        {
                            spriteBatch.DrawString(font, stats[t].ToString(), statLocation2, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f);
                            statLocation2 = statLocation2 + increment; //increment downwards
                        }

                        spriteBatch.DrawString(largeFont, unit.Hp.ToString(), new Vector2(893, 532), Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws unit HP
                        spriteBatch.Draw(unit.getCharInfo(), infoLocation, null, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.7f); //draw charInfoBackground texture
                    }
                }

                #endregion
            }

            #region Game Over
            if (GameState.gameOver)
            {
                Vector2 gameOverLocation = new Vector2(-370, -300);
                spriteBatch.DrawString(largestFont, "Game Over", new Vector2(350, 200), Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0f); //draws Game Over Text
                spriteBatch.Draw(gameOver, Vector2.Zero, null, Color.White, 0, gameOverLocation, 1f, SpriteEffects.None, 0f);
                spriteBatch.Draw(backGround, Vector2.Zero, null, Color.Black * 0.5f, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.9f);

            }
            

            #endregion

            if (GameState.transitionTurn)
            {
                if (GameState.currentPlayer == player1)
                {
                    spriteBatch.Draw(player1Transition, Vector2.Zero, null, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.7f); //draw turn transition 
                }
                else if (GameState.currentPlayer == player2)
                {
                    spriteBatch.Draw(player2Transition, Vector2.Zero, null, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.7f); //draw turn transition 
                }
            }

            spriteBatch.End();

            base.Draw(gameTime);    // repeatedly calls draw
        }

        private static bool findAttackType(Unit unit1)
        {
            if ((unit1.getClass() == UnitType.Warrior) || (unit1.getClass() == UnitType.Archer))
            {
                return false;
            }
            else
            {
                return true;
            }
        }

        //method to return a unit: takes in unit type (warrior/mage/ranger), unit position, and which player's unit (since different model/colors)
        private Unit getNewUnit(UnitType unitType, Vector2 unitPosition, Player player)
        {
            Button[] unitButtons = new Button[9];
            unitButtons[0] = new Button(ButtonType.Attack, unitPosition, Content.Load<Texture2D>("attack"));
            unitButtons[1] = new Button(ButtonType.Move, unitPosition, Content.Load<Texture2D>("move"));
            unitButtons[2] = new Button(ButtonType.Items, unitPosition, Content.Load<Texture2D>("items")); ;
            unitButtons[3] = new Button(ButtonType.Wait, unitPosition, Content.Load<Texture2D>("wait")); ;
            unitButtons[4] = new Button(ButtonType.AttackConfirm, new Vector2(328, 130), Content.Load<Texture2D>("confirm_attack"));
            unitButtons[5] = new Button(ButtonType.Inventory1, unitPosition, Content.Load<Texture2D>("Inventory_Button"));
            unitButtons[6] = new Button(ButtonType.Inventory2, unitPosition, Content.Load<Texture2D>("Inventory_Button"));
            unitButtons[7] = new Button(ButtonType.Inventory3, unitPosition, Content.Load<Texture2D>("Inventory_Button"));
            unitButtons[8] = new Button(ButtonType.Inventory4, unitPosition, Content.Load<Texture2D>("Inventory_Button"));

            // creates human-like characters for current player
            if (player == GameState.currentPlayer)
            {
                if (unitType == UnitType.Warrior)
                {
                    Unit unit = new Warrior(Content.Load<Texture2D>("warrior"), unitButtons, Content.Load<Texture2D>("warrior_stats"), Content.Load<Texture2D>("warrior_attack"), unitPosition);
                    Weapon startingWeap = new BronzeSword();
                    Weapon startingWeap2 = new IronSword();
                    unit.equippedWeapon = startingWeap;
                    unit.getButtonType(ButtonType.Inventory1).weapon = startingWeap;
                    unit.getButtonType(ButtonType.Inventory2).weapon = startingWeap2;
                    graph.getNode(unitPosition).unitOnNode = (unit);
                    return unit;
                }

                if (unitType == UnitType.Mage)
                {
                    Unit unit = new Mage(Content.Load<Texture2D>("mage"), unitButtons, Content.Load<Texture2D>("mage_stats"), Content.Load<Texture2D>("mage_attack"), unitPosition);
                    Weapon startingWeap = new Fireball();
                    Weapon startingWeap2 = new Fireblast();
                    unit.equippedWeapon = startingWeap;
                    unit.getButtonType(ButtonType.Inventory1).weapon = startingWeap;
                    unit.getButtonType(ButtonType.Inventory2).weapon = startingWeap2;
                    graph.getNode(unitPosition).unitOnNode = (unit);
                    return unit;
                }

                if (unitType == UnitType.Archer)
                {
                    Unit unit = new Archer(Content.Load<Texture2D>("archer"), unitButtons, Content.Load<Texture2D>("archer_stats"), Content.Load<Texture2D>("archer_attack"), unitPosition);
                    Weapon startingWeap = new ShortBow();
                    Weapon startingWeap2 = new LongBow();
                    unit.equippedWeapon = startingWeap;
                    unit.getButtonType(ButtonType.Inventory1).weapon = startingWeap;
                    unit.getButtonType(ButtonType.Inventory2).weapon = startingWeap2;
                    graph.getNode(unitPosition).unitOnNode = (unit);
                    return unit;
                }
            }

            // creates ghoul-like characters for enemy player
            if (player == GameState.enemyPlayer)
            {
                if (unitType == UnitType.Warrior)
                {
                    Unit unit = new Warrior(Content.Load<Texture2D>("2warrior"), unitButtons, Content.Load<Texture2D>("2warrior_stats"), Content.Load<Texture2D>("2warrior_attack"), unitPosition);
                    Weapon startingWeap = new BronzeSword();
                    Weapon startingWeap2 = new IronSword();
                    unit.equippedWeapon = startingWeap;
                    unit.getButtonType(ButtonType.Inventory1).weapon = startingWeap;
                    unit.getButtonType(ButtonType.Inventory2).weapon = startingWeap2;
                    graph.getNode(unitPosition).unitOnNode = (unit);
                    return unit;
                }

                if (unitType == UnitType.Mage)
                {
                    Unit unit = new Mage(Content.Load<Texture2D>("2mage"), unitButtons, Content.Load<Texture2D>("2mage_stats"), Content.Load<Texture2D>("2mage_attack"), unitPosition);
                    Weapon startingWeap = new Fireball();
                    Weapon startingWeap2 = new Fireblast();
                    unit.equippedWeapon = startingWeap;
                    unit.getButtonType(ButtonType.Inventory1).weapon = startingWeap;
                    unit.getButtonType(ButtonType.Inventory2).weapon = startingWeap2;
                    graph.getNode(unitPosition).unitOnNode = (unit);
                    return unit;
                }

                if (unitType == UnitType.Archer)
                {
                    Unit unit = new Archer(Content.Load<Texture2D>("2archer"), unitButtons, Content.Load<Texture2D>("2archer_stats"), Content.Load<Texture2D>("2archer_attack"), unitPosition);
                    Weapon startingWeap = new ShortBow();
                    Weapon startingWeap2 = new LongBow();
                    unit.equippedWeapon = startingWeap;
                    unit.getButtonType(ButtonType.Inventory1).weapon = startingWeap;
                    unit.getButtonType(ButtonType.Inventory2).weapon = startingWeap2;
                    graph.getNode(unitPosition).unitOnNode = (unit);
                    return unit;
                }
            }

            return null;
        }

        // manually sets obstacle nodes for graph
        private void setObstacles(Graph graph)
        {
            Texture2D obstacleMap = Content.Load<Texture2D>("map1_obstacles");  // get the obstacle map
            Color[] pixelColor = new Color[1];                                  // holds the colour of the node
            Rectangle sourceRectangle;

            // set node as obstacle if the node on the obstacle map is black
            for (int x = 0; x < obstacleMap.Width; x += 32)
            {
                for (int y = 0; y < obstacleMap.Height; y += 32)
                {
                    sourceRectangle = new Rectangle(x, y, 1, 1);
                    obstacleMap.GetData<Color>(0, sourceRectangle, pixelColor, 0, 1);   // get colour of the node (on obstacle map)
                    if (pixelColor[0] == Color.Black)               // if black, set node as an obstacle
                    {
                        graph.getNode(x / 32, y / 32).isObstacle = true;
                    }
                }
            }
        }
    }
}